<?php
// +-----------------------------------------------------------------+
// |                   PhreeBooks Open Source ERP                    |
// +-----------------------------------------------------------------+
// | Copyright (c) 2008, 2009, 2010 PhreeSoft, LLC                   |
// | http://www.PhreeSoft.com                                        |
// +-----------------------------------------------------------------+
// | This program is free software: you can redistribute it and/or   |
// | modify it under the terms of the GNU General Public License as  |
// | published by the Free Software Foundation, either version 3 of  |
// | the License, or any later version.                              |
// |                                                                 |
// | This program is distributed in the hope that it will be useful, |
// | but WITHOUT ANY WARRANTY; without even the implied warranty of  |
// | MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the   |
// | GNU General Public License for more details.                    |
// |                                                                 |
// | The license that is bundled with this package is located in the |
// | file: /doc/manual/ch01-Introduction/license.html.               |
// | If not, see http://www.gnu.org/licenses/                        |
// +-----------------------------------------------------------------+
//  Path: /modules/phreebooks/functions/phreebooks.php
//

function fetch_item_description($id) {
  global $db;
  $result = $db->Execute("select description from " . TABLE_JOURNAL_ITEM . " where ref_id = " . $id . " limit 1");
  return $result->fields['description'];
}

function validate_fiscal_year($next_fy, $next_period, $next_start_date, $num_periods = 12) {
  global $db;
  for ($i = 0; $i < $num_periods; $i++) {
	$fy_array = array(
	  'period'      => $next_period,
	  'fiscal_year' => $next_fy,
	  'start_date'  => $next_start_date,
	  'end_date'    => gen_specific_date($next_start_date, $day_offset = -1, $month_offset = 1),
	  'date_added'  => date('Y-m-d'),
	);
	db_perform(TABLE_ACCOUNTING_PERIODS, $fy_array, 'insert');
	$next_period++;
	$next_start_date = gen_specific_date($next_start_date, $day_offset = 0, $month_offset = 1);
  }
  return $next_period--;
}

function modify_account_history_records($id, $add_acct = true) {
  global $db;
  $result = $db->Execute("select max(period) as period from " . TABLE_ACCOUNTING_PERIODS);
  $max_period = $result->fields['period'];
  if (!$max_period) die ('table: accounting_periods is not set, run setup.');
  if ($add_acct) {
    $result = $db->Execute("select heading_only from " . TABLE_CHART_OF_ACCOUNTS . " where id = '" . $id . "'");
	if ($result->fields['heading_only'] <> '1') {
	  for ($i = 0, $j = 1; $i < $max_period; $i++, $j++) {
	    $db->Execute("insert into " . TABLE_CHART_OF_ACCOUNTS_HISTORY . " (account_id, period) values('" . $id . "', '" . $j . "')");
	  }
	}
  } else {
	$result = $db->Execute("delete from " . TABLE_CHART_OF_ACCOUNTS_HISTORY . " where account_id = '" . $id . "'");
  }
}

function build_and_check_account_history_records() {
  global $db;
  $result = $db->Execute("select max(period) as period from " . TABLE_ACCOUNTING_PERIODS);
  $max_period = $result->fields['period'];
  if (!$max_period) die ('table: accounting_periods is not set, run setup.');
  $result = $db->Execute("select id, heading_only from " . TABLE_CHART_OF_ACCOUNTS . " order by id");
  while (!$result->EOF) {
    if ($result->fields['heading_only'] <> '1') {
	  $account_id = $result->fields['id'];
	  for ($i = 0, $j = 1; $i < $max_period; $i++, $j++) {
	    $record_found = $db->Execute("select id from " . TABLE_CHART_OF_ACCOUNTS_HISTORY . " where account_id = '" . $account_id . "' and period = " . $j);
	    if (!$record_found->RecordCount()) {
		  $db->Execute("insert into " . TABLE_CHART_OF_ACCOUNTS_HISTORY . " (account_id, period) values('" . $account_id . "', '" . $j . "')");
		 }
	  }
	}
	$result->MoveNext();
  }
}

function get_fiscal_year_pulldown() {
    global $db;
    $fy_values = $db->Execute("select distinct fiscal_year from " . TABLE_ACCOUNTING_PERIODS . " order by fiscal_year");
    $fy_array = array();
    while (!$fy_values->EOF) {
      $fy_array[] = array('id' => $fy_values->fields['fiscal_year'], 'text' => $fy_values->fields['fiscal_year']);
      $fy_values->MoveNext();
    }
    return $fy_array;
}

function load_coa_types() {
  global $coa_types_list;
  if (!is_array($coa_types_list)) { 
    require_once(DIR_FS_MODULES . 'phreebooks/defaults.php');
  }
  $coa_types = array();
  foreach ($coa_types_list as $value) {
    $coa_types[$value['id']] = array(
	  'id'    => $value['id'],
	  'text'  => $value['text'],
	  'asset' => $value['asset'],
	);
  }
  return $coa_types;
}

function load_coa_info($types = array()) { // includes inactive accounts
  global $db;
  $coa_data = array();
  $sql = "select * from " . TABLE_CHART_OF_ACCOUNTS;
  if (sizeof($types > 0)) $sql .= " where account_type in (" . implode(", ", $types) . ")";
  $result = $db->Execute($sql);
  while (!$result->EOF) {
    $coa_data[$result->fields['id']] = array(
	  'id'              => $result->fields['id'],
	  'description'     => $result->fields['description'],
	  'heading_only'    => $result->fields['heading_only'],
	  'primary_acct_id' => $result->fields['primary_acct_id'],
	  'account_type'    => $result->fields['account_type'],
	);
	$result->MoveNext();
  }
  return $coa_data;
}

function fill_paid_invoice_array($id, $account_id, $type = 'c') {
	// to build this data array, all current open invoices need to be gathered and then the paid part needs
	// to be applied along with discounts taken by row.
	global $db, $currencies;
	$negate = ((JOURNAL_ID == 20 && $type == 'c') || (JOURNAL_ID == 18 && $type == 'v')) ? true : false;

	// first read all currently open invoices and the payments of interest and put into an array
	$sql = "select distinct so_po_item_ref_id from " . TABLE_JOURNAL_ITEM . " where ref_id = " . $id;
	$result = $db->Execute($sql);
	$paid_indeces = array();
	while (!$result->EOF) {
		if ($result->fields['so_po_item_ref_id']) $paid_indeces[] = $result->fields['so_po_item_ref_id'];
		$result->MoveNext();
	}
	switch ($type) {
		case 'c': $search_journal = '(12, 13)'; break;
		case 'v': $search_journal = '(6, 7)';   break;
		default: return false;
	}
	$sql = "select id, journal_id, post_date, terms, purch_order_id, purchase_invoice_id, total_amount, gl_acct_id 
		from " . TABLE_JOURNAL_MAIN . " 
		where (journal_id in " . $search_journal . " and closed = '0' and bill_acct_id = " . $account_id . ")";
	if (sizeof($paid_indeces) > 0) $sql .= " or (id in (" . implode(',',$paid_indeces) . ") and closed = '0')";
	$sql .= " order by post_date";
	$result = $db->Execute($sql);
	$open_invoices = array();
	while (!$result->EOF) {
		if ($result->fields['journal_id'] == 7 || $result->fields['journal_id'] == 13) {
		  $result->fields['total_amount'] = -$result->fields['total_amount'];
		}
		$result->fields['total_amount'] -= fetch_partially_paid($result->fields['id']);
		$result->fields['description']   = $result->fields['purch_order_id'];
		$result->fields['discount']      = '';
		$result->fields['amount_paid']   = '';
		$open_invoices[$result->fields['id']] = $result->fields;
		$result->MoveNext();
	}
	// next read the record of interest and add/adjust open invoice array with amounts
	$sql = "select id, ref_id, so_po_item_ref_id, gl_type, description, debit_amount, credit_amount, gl_account 
		from " . TABLE_JOURNAL_ITEM . " where ref_id = " . $id;
	$result = $db->Execute($sql);
	while (!$result->EOF) {
		$amount = ($result->fields['debit_amount']) ? $result->fields['debit_amount'] : $result->fields['credit_amount'];
		if ($negate) $amount = -$amount;
		$index = $result->fields['so_po_item_ref_id'];
		switch ($result->fields['gl_type']) {
		  case 'dsc': // it's the discount field
			$open_invoices[$index]['discount']      = $amount;
			$open_invoices[$index]['amount_paid']  -= $amount;
			break;
		  case 'chk':
		  case 'pmt': // it's the payment field
			$open_invoices[$index]['total_amount'] += $amount;
			$open_invoices[$index]['description']   = $result->fields['description'];
			$open_invoices[$index]['amount_paid']   = $amount;
			break;
		  case 'ttl':
			$payment_fields = $result->fields['description']; // payment details
		  default:
		}
		$result->MoveNext();
	}
	ksort($open_invoices);

	$balance   = 0;
	$index     = 0;
	$item_list = array();
	foreach ($open_invoices as $key => $line_item) {
		// fetch some information about the invoice
		$sql = "select id, post_date, terms, purchase_invoice_id, purch_order_id, gl_acct_id, waiting  
			from " . TABLE_JOURNAL_MAIN . " where id = " . $key;
		$result = $db->Execute($sql);
		$due_dates = calculate_terms_due_dates($result->fields['post_date'], $result->fields['terms'], ($type == 'v' ? 'AP' : 'AR'));
		if ( $negate) {
		  $line_item['total_amount'] = -$line_item['total_amount'];
		  $line_item['discount']     = -$line_item['discount'];
		  $line_item['amount_paid']  = -$line_item['amount_paid'];
		}
		$balance += $line_item['total_amount'];

		$item_list[] = array(
			'id'                  => $result->fields['id'],
			'waiting'             => $result->fields['waiting'],
			'purchase_invoice_id' => $result->fields['purchase_invoice_id'],
			'purch_order_id'      => $result->fields['purch_order_id'],
			'percent'             => $due_dates['discount'],
			'early_date'          => gen_locale_date($due_dates['early_date']),
			'net_date'            => gen_locale_date($due_dates['net_date']),
			'total_amount'        => $currencies->format($line_item['total_amount']),
			'gl_acct_id'          => $result->fields['gl_acct_id'],
			'description'         => $line_item['description'],
			'discount'            => $line_item['discount']    ? $currencies->format($line_item['discount']) : '',
			'amount_paid'         => $line_item['amount_paid'] ? $currencies->format($line_item['amount_paid']) : '',
		);
		$index++;
	}
    return array('balance' => $balance, 'payment_fields' => $payment_fields, 'invoices' => $item_list);
}

function fetch_partially_paid($id, $journal = JOURNAL_ID) {
	global $db;
	$sql = "select sum(i.debit_amount) as debit, sum(i.credit_amount) as credit 
		from " . TABLE_JOURNAL_MAIN . " m inner join " . TABLE_JOURNAL_ITEM . " i on m.id = i.ref_id 
		where m.journal_id in (18, 20) and i.so_po_item_ref_id = " . $id . " group by m.journal_id";
	$result = $db->Execute($sql);
	if ($result->fields['debit']) {
		return $result->fields['debit'];
	} elseif ($result->fields['credit']) {
		return $result->fields['credit'];
	} else {
		return 0;
	}
}

function calculate_terms_due_dates($post_date, $terms_encoded, $type = 'AR') {
  $terms = explode(':', $terms_encoded);
  $date_details = gen_get_dates($post_date);
  $result = array();
  switch ($terms[0]) {
	default:
	case '0': // Default terms
		$result['discount'] = constant($type . '_PREPAYMENT_DISCOUNT_PERCENT') / 100;
		$result['net_date'] = gen_specific_date($post_date, constant($type . '_NUM_DAYS_DUE'));
		if ($result['discount'] <> 0) {
		  $result['early_date'] = gen_specific_date($post_date, constant($type . '_PREPAYMENT_DISCOUNT_DAYS'));
		} else {
		  $result['early_date'] = gen_specific_date($post_date, 1000); // move way out
		}
		break;
	case '1': // Cash on Delivery (COD)
	case '2': // Prepaid
		$result['discount']   = 0;
		$result['early_date'] = $post_date;
		$result['net_date']   = $post_date;
		break;
	case '3': // Special terms
		$result['discount']   = $terms[1] / 100;
		$result['early_date'] = gen_specific_date($post_date, $terms[2]);
		$result['net_date']   = gen_specific_date($post_date, $terms[3]);
		break;
	case '4': // Due on day of next month
		$result['discount']   = $terms[1] / 100;
		$result['early_date'] = gen_specific_date($post_date, $terms[2]);
		$result['net_date']   = gen_db_date( $terms[3] );
		break;
	case '5': // Due at end of month
		$result['discount']   = $terms[1] / 100;
		$result['early_date'] = gen_specific_date($post_date, $terms[2]);
		$result['net_date']   = date('Y-m-d', mktime(0, 0, 0, $date_details['ThisMonth'], $date_details['TotalDays'], $date_details['ThisYear']));
		break;
  }
  return $result;
}

function load_cash_acct_balance($post_date, $gl_acct_id, $period) {
  global $db, $messageStack;
  $acct_balance = 0;
  if (!$gl_acct_id) return $acct_balance;
  $sql = "select beginning_balance from " . TABLE_CHART_OF_ACCOUNTS_HISTORY . " 
	where account_id = '" . $gl_acct_id . "' and period = " . $period;
  $result = $db->Execute($sql);
  $acct_balance = $result->fields['beginning_balance'];

  // load the payments and deposits for the current period
  $bank_list = array();
  $sql = "select i.debit_amount, i.credit_amount
	from " . TABLE_JOURNAL_MAIN . " m inner join " . TABLE_JOURNAL_ITEM . " i on m.id = i.ref_id
	where m.period = " . $period . " and i.gl_account = '" . $gl_acct_id . "' and m.post_date <= '" . $post_date . "' 
	order by m.post_date, m.journal_id";
  $result = $db->Execute($sql);
  while (!$result->EOF) {
    $acct_balance += $result->fields['debit_amount'] - $result->fields['credit_amount'];
    $result->MoveNext();
  }
  return $acct_balance;
}

  function gen_build_tax_auth_array() {
    global $db;
    $tax_auth_values = $db->Execute("select tax_auth_id, description_short, account_id , tax_rate
      from " . TABLE_TAX_AUTH . " order by description_short");
    if ($tax_auth_values->RecordCount() < 1) {
      return false;
    } else {
		while (!$tax_auth_values->EOF) {
		  $tax_auth_array[$tax_auth_values->fields['tax_auth_id']] = array(
			'description_short' => $tax_auth_values->fields['description_short'],
			'account_id'        => $tax_auth_values->fields['account_id'],
			'tax_rate'          => $tax_auth_values->fields['tax_rate'],
		  );
		  $tax_auth_values->MoveNext();
		}
    	return $tax_auth_array;
    }
  }

  function gen_calculate_tax_rate($tax_authorities_chosen, $tax_auth_array) {
	$chosen_auth_array = explode(':', $tax_authorities_chosen);
	$total_tax_rate = 0;
	while ($chosen_auth = array_shift($chosen_auth_array)) {
	  $total_tax_rate += $tax_auth_array[$chosen_auth]['tax_rate'];
	}
	return $total_tax_rate;
  }

  function ord_calculate_tax_drop_down($type = 'c') {
    global $db;
	$tax_auth_array = gen_build_tax_auth_array();
    $sql = "select tax_rate_id, description_short, rate_accounts from " . TABLE_TAX_RATES;
	switch ($type) {
	  default:
	  case 'c':
	  case 'v': $sql .= " where type = '" . $type . "'"; break;
	  case 'b': // both
	}
	$tax_rates = $db->Execute($sql);
    $tax_rate_drop_down = array();
    $tax_rate_drop_down[] = array('id' => '0', 'rate' => '0', 'text' => TEXT_NONE, 'auths' => '');
	while (!$tax_rates->EOF) {
	  $tax_rate_drop_down[] = array(
	    'id'    => $tax_rates->fields['tax_rate_id'],
		'rate'  => gen_calculate_tax_rate($tax_rates->fields['rate_accounts'], $tax_auth_array), 
		'text'  => $tax_rates->fields['description_short'],
		'auths' => $tax_rates->fields['rate_accounts'],
	  );
	  $tax_rates->MoveNext();
	}
	return $tax_rate_drop_down;
  }

  function ord_get_so_po_num($id = '') {
	global $db;
	$result = $db->Execute("select purchase_invoice_id from " . TABLE_JOURNAL_MAIN . " where id = " . $id);
	return ($result->RecordCount()) ? $result->fields['purchase_invoice_id'] : '';
  }

  function ord_get_shipping_methods($method_dir, $methods) {
    global $shipping_defaults;
	$choices = array_keys($shipping_defaults['service_levels']);
	$service_levels = 'var freightLevels = new Array(' . sizeof($choices) . '); ' . chr(10);
	for ($i = 0; $i < sizeof($choices); $i++) {
	  $service_levels .= "freightLevels[" . $i . "]='" . $choices[$i] . "'; " . chr(10);
	}
	$carriers        = 'var freightCarriers = new Array(' . sizeof($methods) . '); ' . chr(10);
	$carrier_details = 'var freightDetails = new Array('  . sizeof($methods) . '); ' . chr(10);
	$i = 0;
	if (sizeof($methods) > 0) {
	  foreach ($methods as $method) {
		$carriers .= "freightCarriers[" . $i . "]='" . $method . "';" . chr(10);
		include_once($method_dir . $method . '/' . $method . '.php');
		$carrier_details .= 'freightDetails[' . $i . '] = new Array(' . sizeof($choices) . '); ' . chr(10);
		for ($j = 0; $j < sizeof($choices); $j++) {
		  $carrier_details .= "freightDetails[" . $i . "][" . $j . "]='" . (defined($method . '_' . $choices[$j]) ? constant($method . '_' . $choices[$j]) : "") . "'; " . chr(10);
		}
	    $i++;
	  }
	}
	return $service_levels . $carriers . $carrier_details;
  }

  function ord_get_projects() {
    global $db;
    $result_array = array();
    $result_array[] = array('id' => '', 'text' => TEXT_NONE);
	// fetch cost structure
	$costs = array();
	$result = $db->Execute("select cost_id, description_short from " . TABLE_PROJECTS_COSTS . " where inactive = '0'");
	while(!$result->EOF) {
	  $costs[$result->fields['cost_id']] = $result->fields['description_short'];
	  $result->MoveNext();
	}
	// fetch phase structure
	$phases = array();
	$result = $db->Execute("select phase_id, description_short, cost_breakdown from " . TABLE_PROJECTS_PHASES . " where inactive = '0'");
	while(!$result->EOF) {
	  $phases[$result->fields['phase_id']] = array(
	  	'text'   => $result->fields['description_short'],
		'detail' => $result->fields['cost_breakdown'],
	  );
	  $result->MoveNext();
	}
	// fetch projects
	$result = $db->Execute("select id, short_name, account_number from " . TABLE_CONTACTS . " where type = 'j' and inactive <> '1'");
	while(!$result->EOF) {
	  $base_id   = $result->fields['id'];
	  $base_text = $result->fields['short_name'];
	  if ($result->fields['account_number'] == '1' && sizeof($phases) > 0) { // use phases
		foreach ($phases as $phase_id => $phase) {
		  $phase_base = $base_id   . ':' . $phase_id;
		  $phase_text = $base_text . ' -> ' . $phase['text'];
		  if ($phase['detail'] == '1' && sizeof($costs) > 0) {
		    foreach ($costs as $cost_id => $cost) {
              $result_array[] = array('id' => $phase_base . ':' . $cost_id, 'text' => $phase_text . ' -> ' . $cost);
			}
		  } else {
            $result_array[] = array('id' => $phase_base, 'text' => $phase_text);
		  }
		}
	  } else {
        $result_array[] = array('id' => $base_id, 'text' => $base_text);
	  }
	  $result->MoveNext();
	}
    return $result_array;
  }

  function gen_auto_update_period($show_message = true) {
	global $db, $messageStack;
	$period = gen_calculate_period(date('Y-m-d'), true);
	if ($period == CURRENT_ACCOUNTING_PERIOD) return; // we're in the current period
	if (!$period) { // we're outside of the defined fiscal years
	  if ($show_message) $messageStack->add(ERROR_MSG_POST_DATE_NOT_IN_FISCAL_YEAR,'error');
	} else { // update CURRENT_ACCOUNTING_PERIOD constant with this new period
	  $result = $db->Execute("select start_date, end_date from " . TABLE_ACCOUNTING_PERIODS . " where period = " . $period);
	  write_configure('CURRENT_ACCOUNTING_PERIOD',       $period);
	  write_configure('CURRENT_ACCOUNTING_PERIOD_START', $result->fields['start_date']);
	  write_configure('CURRENT_ACCOUNTING_PERIOD_END',   $result->fields['end_date']);
	  gen_add_audit_log(GEN_LOG_PERIOD_CHANGE);
	  if ($show_message) {
	    $messageStack->add(sprintf(ERROR_MSG_ACCT_PERIOD_CHANGE, $period),'success');
	  }
	}
  }

  function build_search_sql($fields, $id, $id_from = '', $id_to = '') {
    $crit = array();
    foreach ($fields as $field) {	
	  $output = '';
	  switch ($id) {
	    default:
		case 'all':  break;
		case 'eq':   if ($id_from) $output .= $field . " = '" . $id_from . "'";      break;
		case 'neq':  if ($id_from) $output .= $field . " <> '" . $id_from . "'";     break;
		case 'like': if ($id_from) $output .= $field . " like '%" . $id_from . "%'"; break;
		case 'rng':
		  if ($id_from)          $output .= $field . " >= '" . $id_from . "'";
		  if ($output && $id_to) $output .= " and ";
		  if ($id_to)            $output .= $field . " <= '" . $id_to . "'";
	  }
	  if ($output) $crit[] = $output;
    }
    return ($crit) ? ('(' . implode(' or ', $crit) . ')') : '';
  }

  function repost_journals($journals, $start_date, $end_date) {
	global $db, $messageStack;
	if (sizeof($journals) > 0) {
	  $sql = "select id from " . TABLE_JOURNAL_MAIN . " 
		where journal_id in (" . implode(',', $journals) . ") 
		and post_date >= '" . $start_date . "' and post_date < '" . gen_specific_date($end_date, 1) . "' 
		order by id";
	  $result = $db->Execute($sql);
	  $cnt = 0;
	  $db->transStart();
	  while (!$result->EOF) {
	    $gl_entry = new journal($result->fields['id']);
	    $gl_entry->remove_cogs_rows(); // they will be regenerated during the re-post
	    if (!$gl_entry->Post('edit', true)) {
		  $db->transRollback();
		  $messageStack->add('<br /><br />Failed Re-posting the journals, try a smaller range. The record that failed was # ' . $gl_entry->id,'error');
		  return false;
	    }
		$cnt++;
	    $result->MoveNext();
	  }
      $db->transCommit();
	  return $cnt;
	}
  }

  function validate_gl_balances($action) {
	global $db, $currencies, $messageStack;
	$fiscal_years = array();
	$sql = "select distinct fiscal_year, min(period) as first_period, max(period) as last_period
	  from " . TABLE_ACCOUNTING_PERIODS . " group by fiscal_year order by fiscal_year ASC";
	$result = $db->Execute($sql);
	while (!$result->EOF) {
	  $fiscal_years[] = array(
	    'fiscal_year'  => $result->fields['fiscal_year'],
		'first_period' => $result->fields['first_period'],
		'last_period'  => $result->fields['last_period']);
	  $result->MoveNext();
	}
	$result = $db->Execute("select id from " . TABLE_CHART_OF_ACCOUNTS . " where account_type = 44");
	$retained_earnings_acct = $result->fields['id'];

	$beg_bal = array();
	$bad_accounts = array();
	foreach ($fiscal_years as $fiscal_year) {
	  $sql = "select account_id, period, beginning_balance, (beginning_balance + debit_amount - credit_amount) as next_beg_bal
		from " . TABLE_CHART_OF_ACCOUNTS_HISTORY . " 
		where period >= " . $fiscal_year['first_period'] . " and period <= " . $fiscal_year['last_period'] . " 
		order by period, account_id";
	  $result = $db->Execute($sql);
	  while (!$result->EOF) {
		$period       = $result->fields['period'];
		$next_period  = $period + 1;
		$gl_account   = $result->fields['account_id'];
		$beg_balance  = $currencies->format($result->fields['beginning_balance']);
		$next_beg_bal = $currencies->format($result->fields['next_beg_bal']);
		$beg_bal[$next_period][$gl_account] = $next_beg_bal;
		if ($period <> 1 && $beg_bal[$period][$gl_account] <> $beg_balance) {
		  if ($action <> 'coa_hist_fix') $messageStack->add(sprintf(GEN_ADM_TOOLS_REPAIR_ERROR_MSG, $period, $gl_account, $beg_bal[$period][$gl_account], $beg_balance),'caution');
		  $bad_accounts[$period][$gl_account] = array('sync' => '1');
		}
		// check posted transactions to account to see if they match
		$posted = $db->Execute("select sum(debit_amount) as debit, sum(credit_amount) as credit 
		  from " . TABLE_JOURNAL_MAIN . " m join " . TABLE_JOURNAL_ITEM . " i on m.id = i.ref_id
		  where period = " . $period . " and gl_account = '" . $gl_account . "' 
		  and journal_id in (2, 6, 7, 12, 13, 14, 16, 18, 19, 20, 21)");
		$posted_bal   = $currencies->format($result->fields['beginning_balance'] + $posted->fields['debit'] - $posted->fields['credit']);
		if ($posted_bal <> $next_beg_bal) {
		  if ($action <> 'coa_hist_fix') $messageStack->add(sprintf(GEN_ADM_TOOLS_REPAIR_ERROR_MSG, $period, $gl_account, $posted_bal, $next_beg_bal),'caution');
		  $bad_accounts[$period][$gl_account] = array(
		    'sync'   => '1',
		    'debit'  => $posted->fields['debit'],
		    'credit' => $posted->fields['credit'],
		  );
		}
		$result->MoveNext();
	  }
	  // roll the fiscal year balances
	  // select list of accounts that need to be closed, adjusted
	  $sql = "select id from " . TABLE_CHART_OF_ACCOUNTS . " where account_type in (30, 32, 34, 42, 44)";
	  $result = $db->Execute($sql);
	  $acct_list = array();
	  while(!$result->EOF) {
		$beg_bal[$next_period][$result->fields['id']] = 0;
		$acct_list[] = $result->fields['id'];
		$result->MoveNext();
	  }
	  // fetch the totals for the closed accounts
	  $sql = "select sum(beginning_balance + debit_amount - credit_amount) as retained_earnings 
		from " . TABLE_CHART_OF_ACCOUNTS_HISTORY . " 
		where account_id in ('" . implode("','",$acct_list) . "') and period = " . $period;
	  $result = $db->Execute($sql);
	  $beg_bal[$next_period][$retained_earnings_acct] = $currencies->format($result->fields['retained_earnings']);
	}
	if ($action == 'coa_hist_fix') {
	  // find the affected accounts
	  if (sizeof($bad_accounts) > 0) {
		// *************** START TRANSACTION *************************
		$db->transStart();
	    $glEntry = new journal();
		$min_period = 999999;
		foreach ($bad_accounts as $period => $acct_array) {
		  foreach ($acct_array as $gl_acct => $value) {
			$min_period = min($period, $min_period); // find first period that has an error
			$glEntry->affected_accounts[$gl_acct] = 1;
			if (isset($value['debit'])) { // the history doesn't match posted data, repair
			  $db->Execute("update " . TABLE_CHART_OF_ACCOUNTS_HISTORY . " 
			    set debit_amount = " . $value['debit'] . ", credit_amount = " . $value['credit'] . " 
			    where period = " . $period . " and account_id = '" . $gl_acct . "'");
			}
		  }
		}
		$min_period = max($min_period, 2); // avoid a crash if min_period is the first period
		if ($glEntry->update_chart_history_periods($min_period - 1)) { // from prior period than the error account
			$db->transCommit();
			$messageStack->add_session(GEN_ADM_TOOLS_REPAIR_COMPLETE,'success');
			gen_add_audit_log(GEN_ADM_TOOLS_REPAIR_LOG_ENTRY);
			gen_redirect(html_href_link(FILENAME_DEFAULT, gen_get_all_get_params(array('action')) . 'action=coa_hist_test', 'SSL'));
		}
	  }
	}
	if (sizeof($bad_accounts) == 0) {
	  $messageStack->add(GEN_ADM_TOOLS_REPAIR_SUCCESS,'success');
	} else {
	  $messageStack->add(GEN_ADM_TOOLS_REPAIR_ERROR,'error');
	}
  }

?>