<?php
// +-----------------------------------------------------------------+
// |                   PhreeBooks Open Source ERP                    |
// +-----------------------------------------------------------------+
// | Copyright (c) 2007-2008 PhreeSoft, LLC                          |
// | http://www.PhreeSoft.com                                        |
// +-----------------------------------------------------------------+
// | This program is free software: you can redistribute it and/or   |
// | modify it under the terms of the GNU General Public License as  |
// | published by the Free Software Foundation, either version 3 of  |
// | the License, or any later version.                              |
// |                                                                 |
// | This program is distributed in the hope that it will be useful, |
// | but WITHOUT ANY WARRANTY; without even the implied warranty of  |
// | MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the   |
// | GNU General Public License for more details.                    |
// |                                                                 |
// | The license that is bundled with this package is located in the |
// | file: /doc/manual/ch01-Introduction/license.html.               |
// | If not, see http://www.gnu.org/licenses/                        |
// +-----------------------------------------------------------------+
//  Path: /modules/phreedom/pages/admin/pre_process.php
//

ini_set('memory_limit','256M');  // Set this big for memory exhausted errors
/**************   Check user security   *****************************/
$security_level = $_SESSION['admin_security'][SECURITY_ID_CONFIGURATION];
if ($security_level == 0) { // no permission to enter page, error and redirect to home page
  $messageStack->add_session(ERROR_NO_PERMISSION, 'error');
  gen_redirect(html_href_link(FILENAME_DEFAULT, '', 'SSL'));
}

/**************  include page specific files    *********************/
gen_pull_language($module);
gen_pull_language($module, 'admin');
gen_pull_language('phreeform');
if (defined('MODULE_PHREEFORM_STATUS')) { 
  require_once(DIR_FS_MODULES . 'phreeform/defaults.php');
  require_once(DIR_FS_MODULES . 'phreeform/functions/phreeform.php');
}
require_once(DIR_FS_WORKING . 'functions/phreedom.php');
require_once(DIR_FS_WORKING . 'classes/backup.php');
require_once(DIR_FS_WORKING . 'classes/install.php');
require_once(DIR_FS_WORKING . 'classes/currency.php');

/**************   page specific initialization  *************************/
$error  = false; 
$action = (isset($_GET['action']) ? $_GET['action'] : $_POST['todo']);
$core_modules = array('phreedom','phreeform','phreebooks','contacts','inventory','phreehelp','payment'); // phreeform first!

// see if installing or removing a module
if (substr($action, 0, 8) == 'install_') {
  $method = substr($action, 8);
  $action = 'install';
} elseif (substr($action, 0, 7) == 'remove_') {
  $method = substr($action, 7);
  $action = 'remove';
}
$install   = new phreedom_admin();
$currency  = new currency();

/***************   Act on the action request   *************************/
switch ($action) {
  case 'install':
  case 'update':
	if ($security_level < 4) {
		$messageStack->add_session(ERROR_NO_PERMISSION,'error');
		gen_redirect(html_href_link(FILENAME_DEFAULT, gen_get_all_get_params(array('action')), 'SSL'));
	}
	// load the module installation class
	if (!file_exists(DIR_FS_MODULES . $method . '/classes/install.php')) {
	  $messageStack->add(sprintf('Looking for the installation script for module %s, but could not locate it. The module cannot be installed!', $method),'error');
	  break;
	}
	gen_pull_language($method, 'admin');
	gen_pull_language($method);
	require_once(DIR_FS_MODULES . $method . '/config.php'); // config is not loaded yet since module is not installed.
	require_once(DIR_FS_MODULES . $method . '/classes/install.php');
	$cName = $method . '_admin';
	$mInstall = new $cName();
	if ($action == 'install') {
	  if (admin_check_versions($method, $mInstall->prerequisites)) { // Check for version levels
	    $error = true;
	  } elseif (admin_install_dirs($mInstall->dirlist, DIR_FS_MY_FILES.$_SESSION['company'].'/')) {
	    $error = true;
	  } elseif (admin_install_tables($mInstall->tables)) { // Create the tables
	    $error = true; 
	  } else {
	    // Load the installed module version into db
	    write_configure('MODULE_' . strtoupper($method) . '_STATUS', constant('MODULE_' . strtoupper($method) . '_VERSION'));
	    // Load the remaining configuration constants
	    foreach ($mInstall->keys as $key => $value) write_configure($key, $value);
	    if ($demo) $error = $mInstall->load_demo(); // load demo data
		$mInstall->load_reports($method);
		admin_add_reports($method);
	  }
	  if ($mInstall->install($method)) $error = true; // install any special stuff
	} else {
	  if ($mInstall->update($method)) $error = true;
	}
	if ($error) break; 
	if (sizeof($mInstall->notes) > 0) foreach ($mInstall->notes as $note) $messageStack->add($note, 'caution');
	gen_add_audit_log(sprintf(GEN_LOG_INSTALL_SUCCESS, $method) . (($action == 'install') ? TEXT_INSTALL : TEXT_UPDATE), constant('MODULE_' . strtoupper($method) . '_VERSION'));
	gen_redirect(html_href_link(FILENAME_DEFAULT, gen_get_all_get_params(array('action')), 'SSL'));
	break;
  case 'remove':
	if ($security_level < 4) {
		$messageStack->add_session(ERROR_NO_PERMISSION,'error');
		gen_redirect(html_href_link(FILENAME_DEFAULT, gen_get_all_get_params(array('action')), 'SSL'));
	}
	// load the module installation class
	if (!file_exists(DIR_FS_MODULES . $method . '/classes/install.php')) {
	  $messageStack->add(sprintf('Looking for the installation script for module %s, but could not locate it. The module cannot be installed!', $method),'error');
	  break;
	}
	require_once(DIR_FS_MODULES . $method . '/classes/install.php');
	$cName = $method . '_admin';
	$mInstall = new $cName();
	foreach ($mInstall->keys as $key => $value) remove_configure($key);
	remove_configure('MODULE_' . strtoupper($method) . '_STATUS');
	if (admin_remove_tables(array_keys($mInstall->tables))) $error = true;
	if (admin_remove_dirs($mInstall->dirlist))              $error = true;
	if ($mInstall->remove($method))                         $error = true;
	gen_add_audit_log(sprintf(AUDIT_LOG_REMOVE_SUCCESS, $module));
	gen_redirect(html_href_link(FILENAME_DEFAULT, gen_get_all_get_params(array('action')), 'SSL'));
	break;
  case 'save':
	if ($security_level < 3) {
		$messageStack->add_session(ERROR_NO_PERMISSION,'error');
		gen_redirect(html_href_link(FILENAME_DEFAULT, gen_get_all_get_params(array('action')), 'SSL')); 
	}
	// save general tab
	foreach ($install->keys as $key => $default) {
	  $field = strtolower($key);
      if (isset($_POST[$field])) write_configure($key, db_prepare_input($_POST[$field]));
	  // special case for field COMPANY_NAME to update company config file
	  if ($key == 'COMPANY_NAME' && $_POST[$field] <> COMPANY_NAME) {
	    if (!install_build_co_config_file($_SESSION['company'], $_SESSION['company'] . '_TITLE', db_prepare_input($_POST[$field]))) $error = true;
	  }
    }
	$messageStack->add(GENERAL_CONFIG_SAVED,'success');
//	gen_redirect(html_href_link(FILENAME_DEFAULT, gen_get_all_get_params(array('action')), 'SSL'));
	$def_tab = 'company';
    break;
  case 'delete':
	if ($security_level < 4) {
	  $messageStack->add_session(ERROR_NO_PERMISSION,'error');
	  break;
	}
    $subject = $_POST['subject'];
    $id      = $_POST['rowSeq'];
	if (!$subject || !$id) break;
    if ($$subject->btn_delete($id)) $close_popup = true;
	break;
  case 'copy_co':
	$db_server = db_prepare_input($_POST['db_server']);
	$db_name   = db_prepare_input($_POST['db_name']);
	$db_user   = db_prepare_input($_POST['db_user']);
	$db_pw     = db_prepare_input($_POST['db_pw']);
	$co_name   = db_prepare_input($_POST['co_name']);
	// error check company name and company full name
	if (!$db_name || !$co_name)           $error = $messageStack->add(SETUP_CO_MGR_ERROR_EMPTY_FIELD,'error');
	if ($db_name == $_SESSION['company']) $error = $messageStack->add(SETUP_CO_MGR_DUP_DB_NAME,'error');
	// check for database already exists
	$db_orig = new queryFactory;
	if (!$db_orig->connect(DB_SERVER_HOST, DB_SERVER_USERNAME, DB_SERVER_PASSWORD, DB_DATABASE)) {
	  die('Problem connecting to original DB!');
	}
	$db = new queryFactory;
	if (!$db->connect($db_server, $db_user, $db_pw, $db_name)) $error = $messageStack->add(SETUP_CO_MGR_CANNOT_CONNECT,'error');

	if (!$error) { // write the db config.php in the company directory
	  if (!file_exists(DIR_FS_ADMIN . PATH_TO_MY_FILES . $db_name)) {
	    if (!@mkdir   (DIR_FS_ADMIN . PATH_TO_MY_FILES . $db_name)) $error = $messageStack->add(sprintf(MSG_ERROR_CREATE_MY_FILES, DIR_FS_ADMIN . PATH_TO_MY_FILES),'error');
	  }
	  if (!install_build_co_config_file($db_name, $db_name . '_TITLE',  $co_name))   $error = true;
	  if (!install_build_co_config_file($db_name, 'DB_SERVER_USERNAME', $db_user))   $error = true;
	  if (!install_build_co_config_file($db_name, 'DB_SERVER_PASSWORD', $db_pw))     $error = true;
	  if (!install_build_co_config_file($db_name, 'DB_SERVER_HOST',     $db_server)) $error = true;
	}

	$copy_modules = $core_modules;
	foreach ($loaded_modules as $entry) if (isset($_POST[$entry])) $copy_modules[] = $entry;
	$backup = new backup;
    $backup->source_dir  = DIR_FS_MY_FILES . $db_name . '/temp/';
    $backup->source_file = 'temp.sql';
	if (!$error) {
	  foreach ($copy_modules as $entry) gen_pull_language($entry, 'admin');
//	  $notes   = array();
	  foreach ($copy_modules as $entry) {
		require_once (DIR_FS_MODULES . $entry . '/classes/install.php');
		$classname   = $entry . '_admin';
		$install_mod = new $classname;
	    $task        = $_POST[$entry . '_action']; 
		if ($entry == 'phreedom') $task = 'data'; // force complete copy of phreedom module
	    switch ($task) {
		  case 'core':
		  case 'demo':
			if (admin_install_dirs($install_mod->dirlist, DIR_FS_MY_FILES . $db_name . '/')) {
			  $error = true;
			} elseif (admin_install_tables($install_mod->tables)) { // Create the tables
			  $error = true; 
			} else {
			  // Load the installed module version into db
			  write_configure('MODULE_' . strtoupper($entry) . '_STATUS', constant('MODULE_' . strtoupper($entry) . '_VERSION'));
			  // Load the remaining configuration constants
			  foreach ($install_mod->keys as $key => $value) write_configure($key, $value);
			  if ($task == 'demo') if ($install_mod->load_demo()) $error = true; // load demo data
			  if ($entry <> 'phreedom') $install_mod->load_reports($entry);
			  if ($entry == 'phreeform') {
			  	$temp_mod = new phreedom_admin;
	  			$temp_mod->load_reports('phreedom');
			  }
			}
			if ($install_mod->install($entry)) $error = true; // install any special stuff
			if ($error) $messageStack->add(sprintf(MSG_ERROR_MODULE_INSTALL, $entry), 'error');
//			if (sizeof($install_mod->notes) > 0) $notes = array_merge($notes, $install_mod->notes);
		    break;
		  default:
		  case 'data':
			$table_list = array();
		    if (is_array($install_mod->tables)) {
			  foreach ($install_mod->tables as $table => $create_sql) $table_list[] = $table;
			  $backup->copy_db_table($db_orig, $table_list, $type = 'both', $params = '');
	  		}
			if (is_array($install_mod->dirlist)) foreach($install_mod->dirlist as $source_dir) {
		      $dir_source = DIR_FS_MY_FILES . $_SESSION['company'] . '/' . $source_dir . '/';
		      $dir_dest   = DIR_FS_MY_FILES . $db_name             . '/' . $source_dir . '/';
			  $backup->copy_dir($dir_source, $dir_dest);
		    }
			break;
		  default: // skip, should not happen
	    }
	  }
	  // install reports now that categories are set up
	  foreach ($copy_modules as $entry) admin_add_reports($entry, DIR_FS_MY_FILES . $db_name . '/phreeform/');
	}
	if (!$error && $_POST['phreebooks_action'] <> 'data') { // install fiscal year if the phreebooks data is not copied
	  $db->Execute("TRUNCATE TABLE " . TABLE_ACCOUNTING_PERIODS);
	  require_once(DIR_FS_MODULES . 'phreebooks/functions/phreebooks.php');
	  $dates = gen_get_dates();
	  validate_fiscal_year($dates['ThisYear'], '1', $dates['ThisYear'] . '-' . $dates['ThisMonth'] . '-01');
	  build_and_check_account_history_records();
	  gen_auto_update_period(false);
	}
	if (!$error) { // reset SESSION['company'] to new company and redirect to install->store_setup
	  $db->Execute("update " . TABLE_CONFIGURATION . " set configuration_value = '" . $co_name . "' 
	    where configuration_key = 'COMPANY_NAME'");
	  $messageStack->add(SETUP_CO_MGR_CREATE_SUCCESS,'success');
	  gen_add_audit_log(SETUP_CO_MGR_LOG . TEXT_COPY, $db_name);
	  $_SESSION['db_server'] = $db_server;
	  $_SESSION['company']   = $db_name;
	  $_SESSION['db_user']   = $db_user;
	  $_SESSION['db_pw']     = $db_pw;
      gen_redirect(html_href_link(FILENAME_DEFAULT, $get_parmas, ENABLE_SSL_ADMIN ? 'SSL' : 'NONSSL'));
	} else { // restore db connection
	  $db = new queryFactory;
	  $db->connect(DB_SERVER_HOST, DB_SERVER_USERNAME, DB_SERVER_PASSWORD, DB_DATABASE);
	}
	$def_tab = 'manager';
	break;
  case 'delete_co':
	$db_name = $_SESSION['companies'][$_POST['del_company']];
	// Failsafe to prevent current company from being deleted accidently
	$backup = new backup;
	if ($db_name == 'none') {
	  $error = $messageStack->add(SETUP_CO_MGR_NO_SELECTION,'error');
	}
	if (!$error && $db_name <> $_SESSION['company']) {
	  // connect to other company, retrieve login info
	  $config = file(DIR_FS_MY_FILES . $db_name . '/config.php');
	  foreach ($config as $line) {
	    if     (strpos($line, 'DB_SERVER_USERNAME')) $db_user   = substr($line, strpos($line,",")+1, strpos($line,")")-strpos($line,",")-1);
	    elseif (strpos($line, 'DB_SERVER_PASSWORD')) $db_pw     = substr($line, strpos($line,",")+1, strpos($line,")")-strpos($line,",")-1);
	    elseif (strpos($line, 'DB_SERVER_HOST'))     $db_server = substr($line, strpos($line,",")+1, strpos($line,")")-strpos($line,",")-1);
	  }
	  $db_user   = str_replace("'", "", $db_user);
	  $db_pw     = str_replace("'", "", $db_pw);
	  $db_server = str_replace("'", "", $db_server);
	  $del_db = new queryFactory;
	  if (!$del_db->connect($db_server, $db_user, $db_pw, $db_name)) $error = $messageStack->add(SETUP_CO_MGR_CANNOT_CONNECT,'error');
	  if (!$error) {
	    $tables = array();
	    $table_list = $del_db->Execute("show tables");
	    while (!$table_list->EOF) {
		  $tables[] = array_shift($table_list->fields);
		  $table_list->MoveNext();
	    }
	    if (is_array($tables)) foreach ($tables as $table) $del_db->Execute("drop table " . $table);
	    $backup->delete_dir(DIR_FS_MY_FILES . $db_name);
	    unset($_SESSION['companies'][$_POST['del_company']]);
	    gen_add_audit_log(SETUP_CO_MGR_LOG . TEXT_DELETE, $db_name);
	    $messageStack->add(SETUP_CO_MGR_DELETE_SUCCESS, 'success');
	  }
	}
	$def_tab = 'manager';
	break;
  default:
}

/*****************   prepare to display templates  *************************/
// load other module admin information
$page_list = array();
if ($dir = @dir(DIR_FS_MODULES)) {
  while ($file = $dir->read()) {
    if (is_dir(DIR_FS_MODULES . $file) && $file <> '.' && $file <> '..') {
	  if (file_exists(DIR_FS_MODULES . $file . '/config.php') && $file <> 'phreedom') {
		gen_pull_language($file, 'admin');
		$page_list[$file] = constant('MODULE_' . strtoupper($file) . '_TITLE');
	  }
    }
  }
  $dir->close();
}
asort($page_list);

// build some general pull down arrays
$sel_yes_no = array(
 array('id' => '0', 'text' => TEXT_NO),
 array('id' => '1', 'text' => TEXT_YES),
);

$sel_transport = array(
  array('id' => 'PHP',        'text' => 'PHP'),
  array('id' => 'sendmail',   'text' => 'sendmail'),
  array('id' => 'sendmail-f', 'text' => 'sendmail-f'),
  array('id' => 'smtp',       'text' => 'smtp'),
  array('id' => 'smtpauth',   'text' => 'smtpauth'),
  array('id' => 'Qmail',      'text' => 'Qmail'),
);

$sel_linefeed = array(
  array('id' => 'LF',   'text' => 'LF'),
  array('id' => 'CRLF', 'text' => 'CRLF'),
);

$sel_format = array(
  array('id' => 'TEXT', 'text' => 'TEXT'),
  array('id' => 'HTML', 'text' => 'HTML'),
);

$sel_order_lines = array(
  array('id' => '0', 'text' => TEXT_DOUBLE_MODE),
  array('id' => '1', 'text' => TEXT_SINGLE_MODE),
);

$sel_ie_method = array(
  array('id' => 'l', 'text' => TEXT_LOCAL),
  array('id' => 'd', 'text' => TEXT_DOWNLOAD),
);

$include_header   = true;
$include_footer   = true;
$include_tabs     = true;
$include_calendar = false;
$include_template = 'template_main.php';
define('PAGE_TITLE', BOX_GENERAL_ADMIN);

?>